local addon = "QuickCommands"
local author = GetAddOnMetadata(addon, "Author")
local version = GetAddOnMetadata(addon, "Version")

QC_OPTIONS = QC_OPTIONS or {}
QC_OPTIONS['autotrack'] = QC_OPTIONS['autotrack'] or false

local frame = CreateFrame("FRAME", "QuickCommands")
frame:RegisterEvent("QUEST_QUERY_COMPLETE")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
local quests = {}
local autotrackCounter = 0
local autotrackType = 0


local desc = {
	" |cFFFF0000/gx|r or |cFFFF0000/qc gx|r: Restarts graphics engine.",
	" |cFFFF0000/netstats|r or |cFFFF0000/qc netstats|r: Prints netstats.",
	" |cFFFF0000/learn|r or |cFFFF0000/qc learn|r: Learns spells from trainer.",
	" |cFFFF0000/rl|r or |cFFFF0000/reload|r or |cFFFF0000/qc rl|r or |cFFFF0000/qc reload|r: Reloads user interface.",
	" |cFFFF0000/cl|r or |cFFFF0000/qc cl|r: Clears combat log.",
	" |cFFFF0000/mm|r |cff0055FF<macro text>|r or |cFFFF0000/mmg|r |cff0055FF<macro text>|r: Creates global macro.",
	" |cFFFF0000/mmc|r |cff0055FF<macro text>|r: Creates character specific macro.",
	" |cFFFF0000/pvpstatus|r or |cFFFF0000/pvps|r or |cFFFF0000/qc pvpstatus|r or |cFFFF0000/qc pvps|r: Shows PvP status.",
	" |cFFFF0000/q <quest id>|r or |cFFFF0000/quest <quest id>|r or |cFFFF0000/qc q <quest id>|r or |cFFFF0000/qc quest <quest id>|r: Gets quest completion status.",
	" |cFFFF0000/calc|r |cff0055FF<expression>|r or |cFFFF0000/qc calc |cff0055FF<expression>|r: Calculates something.",
	" |cFFFF0000/autotrack|r or |cFFFF0000/qc autotrack|r: Toggles automated switching between Find Herbs and Find Minerals.",
	" |cFFFF0000/helm|r or |cFFFF0000/qc helm|r: Toggles display of helm on character.",
	" |cFFFF0000/cloak|r or |cFFFF0000/qc cloak|r: Toggles display of cloak on character.",
	" |cFFFF0000/guid|r or |cFFFF0000/qc guid|r: Prints target GUID.",
	" |cFFFF0000/combat|r or |cFFFF0000/qc combat|r: Prints party/raid members in combat.",
}

if GetLocale() == "deDE" then
	desc = {
		" |cFFFF0000/gx|r oder |cFFFF0000/qc gx|r: Startet die Grafikengine neu.",
		" |cFFFF0000/netstats|r oder |cFFFF0000/qc netstats|r: Zeigt Netzwerkstatistiken an.",
		" |cFFFF0000/learn|r oder |cFFFF0000/qc learn|r: Lernt alles vom Lehrer.",
		" |cFFFF0000/rl|r oder |cFFFF0000/reload|r oder |cFFFF0000/qc rl|r oder |cFFFF0000/qc reload|r: Lädt das Interface neu.",
		" |cFFFF0000/cl|r oder |cFFFF0000/qc cl|r: Leert das Kampflog.",
		" |cFFFF0000/mm|r |cff0055FF<macro text>|r oder |cFFFF0000/mmg|r |cff0055FF<macro text>|r: Erstellt ein globales Makro.",
		" |cFFFF0000/mmc|r |cff0055FF<macro text>|r: Erstellt ein charakterspezifisches Makro.",
		" |cFFFF0000/pvpstatus|r oder |cFFFF0000/pvps|r oder |cFFFF0000/qc pvpstatus|r oder |cFFFF0000/qc pvps|r: Zeigt PvP-Status an.",
		" |cFFFF0000/q <quest id>|r oder |cFFFF0000/quest <quest id>|r oder |cFFFF0000/qc q <quest id>|r oder |cFFFF0000/qc quest <quest id>|r: Zeigt Queststatus an.",
		" |cFFFF0000/calc|r |cff0055FF<expression>|r oder |cFFFF0000/qc calc |cff0055FF<expression>|r: Rechnet etwas aus.",
		" |cFFFF0000/autotrack|r oder |cFFFF0000/qc autotrack|r: Schaltet den automatischen Wechsel zwischen Kräuter- und Erz-Suche an oder aus.",
		" |cFFFF0000/helm|r oder |cFFFF0000/qc helm|r: Schaltet die Anzeige des Helms an oder aus.",
		" |cFFFF0000/cloak|r oder |cFFFF0000/qc cloak|r: Schaltet die Anzeige des Umhangs an oder aus.",
		" |cFFFF0000/guid|r oder |cFFFF0000/qc guid|r: Zeigt die GUID des Ziels.",
		" |cFFFF0000/combat|r oder |cFFFF0000/qc combat|r: Zeigt den Kampfstatus der Gruppen-/Raid-Mitglieder.",
	}
end

local function p(str)
	if type(str) == "nil" then
		str = "nil"
	elseif type(str) == "boolean" then
		if str then
			str = "true"
		else
			str = "false"
		end
	end
	
	print("|cFF00AACCQC: "..str)
end

local function questLink(id, done)
	local color = ""
	if done then
		color = "|cff40c040"
	else
		color = "|cffff2020"
	end
	return color.."|Hquest:"..id..":0|h["..id.."]|h|r"
end

local function uc(cmd)
	p("|cFFFF0000unrecognized command:|r "..cmd)
end

local function netstats()
	local down, up, latency = GetNetStats()
	p("Downstream: "..down.." KB/s")
	p("Upstream: "..up.." KB/s")
	p("Latency: "..latency.." ms")
end

local function learn()
	LoadAddOn("Blizzard_TrainerUI")
	f = ClassTrainerTrainButton
	f.e = 0
	if f:GetScript("OnUpdate") then
		f:SetScript("OnUpdate", nil)
	else
		f:SetScript("OnUpdate", function(f,e) f.e=f.e+e if f.e>.01 then f.e=0 f:Click() end end)
	end
end

local function gx()
	RestartGx()
end

local function mmg(macro)
	local gmacros, cmacros = GetNumMacros()
	local name = "QC_"..GetTime()
	
	if gmacros < 36 then
		CreateMacro(name, 180, macro, 0, 1)
		p("Macro created: "..name..": "..macro)
	else
		p("No more room for macros!")
	end
end

local function mmc(macro)
	local gmacros, cmacros = GetNumMacros()
	local name = "QC_"..GetTime()
	
	if cmacros < 18 then
		CreateMacro(name, 180, macro, 1, 1)
		p("Macro created: "..name..": "..macro)
	else
		p("No more room for macros!")
	end
end

local function reload()
	ReloadUI()
end

local function combatlog()
	CombatLogClearEntries()
end

local function lick()
	DoEmote("lick")
end

local function pvpstatus()	
	local msg = ""
	if GetZonePVPInfo() == "sanctuary" then
		msg = "PvP: Off (sanctuary)"
	else
		if UnitIsPVP("player") then
			msg = "PvP: On"
			local z = GetZonePVPInfo()
			if z == "arena" or z == "combat" or z == "contested" or z == "hostile" then
				msg = msg.." (PvP Zone)"
			else
				local timer = GetPVPTimer()
				if timer == 301000 or timer == 0 or timer == 999 then
					msg = msg.." (permanent)"
				else
					timer = timer / 1000
					local m = math.floor(timer / 60).."min "
					local s = math.floor(timer % 60).."s"
					msg = msg.." ("..m..s..")"
				end
			end
		else
			msg = "PvP: Off"
		end
	end
	p(msg)
end

local function autotrackTick(elapsed)
	if not QC_OPTIONS['autotrack'] then
		return
	end
	if UnitCastingInfo("player") or GetNumLootItems() > 0 or UnitIsDeadOrGhost("player") then
		return
	end
	
	autotrackCounter = autotrackCounter + elapsed
	if autotrackCounter > 2 then
		autotrackCounter = 0
		SetTracking(autotrackType)
		if autotrackType == 1 then
			autotrackType = 2
		else
			autotrackType = 1
		end
	end
end

local function onUpdate(self, elapsed)
	if QC_OPTIONS['autotrack'] then
		autotrackTick(elapsed)
	end
end

local function eventHandler(self, event, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		frame:SetScript("OnUpdate", onUpdate)
		frame:SetScript("OnEvent", nil)
	elseif event == "QUEST_QUERY_COMPLETE" then
		local t = GetQuestsCompleted()
		while #quests > 0 do
			quest = tonumber(table.remove(quests))
			if t[quest] then
				p(questLink(quest, true)..": true")
			else
				p(questLink(quest, false)..": false")
			end
		end
		frame:SetScript("OnEvent", nil)
	end
end

local function q(str)
	for id in str:gmatch("%d+") do
		table.insert(quests, tonumber(id))
	end
	frame:SetScript("OnEvent", eventHandler)
	QueryQuestsCompleted()
end

local function calc(str)
	local func = loadstring("return "..str)
	p(str.." = "..func())
end

local function autotrack()
	if not IsSpellKnown(2383) or not IsSpellKnown(2580) then
		QC_OPTIONS['autotrack'] = false
		p("Tracking spells missing, Autotrack disabled.")
		return
	end
	QC_OPTIONS['autotrack'] = not QC_OPTIONS['autotrack']
	if QC_OPTIONS['autotrack'] then
		p("Autotrack on.")
	else
		p("Autotrack off.")
	end
end

local function helm()
	ShowHelm(not ShowingHelm())
end

local function cloak()
	ShowCloak(not ShowingCloak())
end

local function guid()
	if not UnitExists("target") then
		p("No target.")
		return
	end
	
	if not UnitIsPlayer("target") then
		p("Not a player.")
		return
	end
	
	local name = UnitName("target")
	local guid = tonumber(UnitGUID("target"), 16)
	p(name..": "..guid)
end

local function combat()
	local channel = ""
	local unitPrefix = ""
	local numMembers = 0
	
	if GetNumRaidMembers() > 0 then
		channel = "RAID"
		unitPrefix = "raid"
		numMembers = 40
	elseif GetNumPartyMembers() > 0 then
		channel = "PARTY"
		unitPrefix = "party"
		numMembers = 5
	end
	
	if channel == "" or unitPrefix == "" or numMembers == 0 then
		p("Not in party/raid")
		return
	end

	local count = 0
	local names = {}
	
	if numMembers == 5 then
		local unit = "player"
		
		if UnitAffectingCombat(unit) then
			count = count + 1
			local name = UnitName(unit)
			if name then
				table.insert(names, name)
			end
		end
		
		if UnitAffectingCombat(unit.."pet") then
			count = count + 1
			local name = UnitName(unit.."pet")
			local ownerName = UnitName(unit)
			if name and ownerName then
				name = name.." ("..ownerName.."'s Pet)"
				table.insert(names, name)
			end
		end
	end
	
	for i = 1, numMembers, 1 do
		local unit = unitPrefix..i
		
		if UnitAffectingCombat(unit) then
			count = count + 1
			local name = UnitName(unit)
			if name then
				table.insert(names, name)
			end
		end
		
		if UnitAffectingCombat(unit.."pet") then
			count = count + 1
			local name = UnitName(unit.."pet")
			local ownerName = UnitName(unit)
			if name and ownerName then
				name = name.." ("..ownerName.."'s Pet)"
				table.insert(names, name)
			end
		end
	end
	
	if count == 0 then
		SendChatMessage("In combat - [0]", channel)
		return
	end
	
	nameString = ""
	for i = 1, #names, 1 do
		nameString = nameString..names[i]
		if i ~= #names then
			nameString = nameString..", "
		end
	end
	
	SendChatMessage("In combat - ["..count.."]: "..nameString, channel)
end

local function qc(str)
	local cmd, a1 = strsplit(" ", str, 2)
	cmd = cmd or ""
	a1 = a1 or ""
	if cmd == "gx" then
		gx()
	elseif cmd == "cl" then
		combatlog()
	elseif cmd == "rl" or cmd == "reload" then
		reload()
	elseif cmd == "netstats" then
		netstats()
	elseif cmd == "learn" then
		learn()
	elseif cmd == "pvpstatus" or cmd == "pvps" then
		pvpstatus()
	elseif cmd == "quest" or cmd == "q" and a1 ~= nil and a1 ~= "" then
		q(a1)
	elseif cmd == "calc" then
		str = str:gsub("^calc ", "")
		str = str:gsub("^calc", "")
		calc(str)
	elseif cmd == "autotrack" then
		autotrack()
	elseif cmd == "helm" then
		helm()
	elseif cmd == "cloak" then
		cloak()
	elseif cmd == "guid" then
		guid()
	elseif cmd == "combat" then
		combat()
	elseif cmd == "about" then
		p(addon.." by "..author)
		p("Version: "..version)
	elseif cmd == "" or cmd == "help" then
		p("Commands:")
		for _, v in ipairs(desc) do
			p(v)
		end
	else
		uc(cmd)
	end
end

frame:SetScript("OnEvent", eventHandler)

SLASH_LEARN1 = "/learn"
SlashCmdList["LEARN"] = learn

SLASH_GX1 = "/gx"
SlashCmdList["GX"] = gx

SLASH_MMG1="/mmg"
SLASH_MMG2="/mm"
SlashCmdList["MMG"] = mmg

SLASH_MMC1="/mmc"
SlashCmdList["MMC"] = mmc

SLASH_CL1="/cl"
SlashCmdList["CL"] = combatlog

SLASH_RL1="/rl"
SLASH_RL2="/reload"
SlashCmdList["RL"] = reload

SLASH_NETSTATS1 = "/netstats"
SlashCmdList["NETSTATS"] = netstats

SLASH_LICK1="/l"
SlashCmdList["LICK"] = lick

SLASH_QC1 = "/qc"
SLASH_QC2 = "/quickcommands"
SlashCmdList["QC"] = qc

SLASH_PVPSTATUS1 = "/pvps"
SLASH_PVPSTATUS2 = "/pvpstatus"
SlashCmdList["PVPSTATUS"] = pvpstatus

SLASH_Q1 = "/quest"
SLASH_Q2 = "/q"
SlashCmdList["Q"] = q

SLASH_CALC1 = "/calc"
SlashCmdList["CALC"] = calc

SLASH_AUTOTRACK1 = "/autotrack"
SlashCmdList["AUTOTRACK"] = autotrack

SLASH_HELM1 = "/helm"
SlashCmdList["HELM"] = helm

SLASH_CLOAK1 = "/cloak"
SlashCmdList["CLOAK"] = cloak

SLASH_GUID1 = "/guid"
SlashCmdList["GUID"] = guid

SLASH_COMBAT1 = "/combat"
SlashCmdList["COMBAT"] = combat